@@ -1,9 +1,28 @@
-/* $Xorg: xhost.c,v 1.4 2001/02/09 02:05:46 xorgcvs Exp $ */
-/* $XdotOrg: app/xhost/xhost.c,v 1.4 2005/06/18 08:03:35 alanc Exp $ */
+/*
+ * Copyright (c) 2004, Oracle and/or its affiliates. All rights reserved.
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
+ */
 /*
 
 Copyright 1985, 1986, 1987, 1998  The Open Group
-Copyright 2004 Sun Microsystems, Inc.
 
 All rights reserved.
 
@@ -35,21 +54,17 @@ of the copyright holder.
 X Window System is a trademark of The Open Group.
 
 */
-/* $XFree86: xc/programs/xhost/xhost.c,v 3.26 2003/07/27 14:05:45 herrb Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#if defined(TCPCONN) || defined(STREAMSCONN)
+#ifdef TCPCONN
 #define NEEDSOCKETS
 #endif
 #ifdef UNIXCONN
 #define NEEDSOCKETS
 #endif
-#ifdef DNETCONN
-#define NEEDSOCKETS
-#endif
 
 #include <X11/Xlib.h>
 #include <X11/Xos.h>
@@ -58,45 +73,25 @@ X Window System is a trademark of The Open Group.
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
-#ifdef X_NOT_POSIX
-#include <setjmp.h>
-#endif
 #include <ctype.h>
 #include <X11/Xauth.h>
 #include <X11/Xmu/Error.h>
 #include <stdlib.h>
 
 #ifdef NEEDSOCKETS
-#ifdef att
-typedef unsigned short unsign16;
-typedef unsigned long unsign32;
-typedef short sign16;
-typedef long sign32;
-#include <interlan/socket.h>
-#include <interlan/netdb.h>
-#include <interlan/in.h>
+#ifdef WIN32
+#include <X11/Xwinsock.h>
 #else
-#ifndef Lynx
 #include <sys/socket.h>
-#else
-#include <socket.h>
-#endif
 #include <netdb.h>
 #include <netinet/in.h>
 #endif
 #endif /* NEEDSOCKETS */
 
-#ifndef BAD_ARPAINET
+#ifndef WIN32
 #include <arpa/inet.h>
-#else
-/* bogus definition of inet_makeaddr() in BSD 4.2 and Ultrix */
-extern unsigned long inet_makeaddr();
 #endif
 
-#ifdef DNETCONN
-#include <netdnet/dn.h>
-#include <netdnet/dnetdb.h>
-#endif
 
 #ifdef SECURE_RPC
 #include <pwd.h>
@@ -106,7 +101,7 @@ extern unsigned long inet_makeaddr();
 #include <limits.h>
 #undef _POSIX_C_SOURCE
 #else
-#if defined(X_NOT_POSIX) || defined(_POSIX_SOURCE)
+#ifdef _POSIX_SOURCE
 #include <limits.h>
 #else
 #define _POSIX_SOURCE
@@ -125,19 +120,9 @@ extern int getdomainname(char *name, size_t len);
 #endif
 
 static int change_host(Display *dpy, char *name, Bool add);
-static char *get_hostname(XHostAddress *ha);
+static const char *get_hostname(XHostAddress *ha);
 static int local_xerror(Display *dpy, XErrorEvent *rep);
-
-#ifdef RETSIGTYPE /* autoconf AC_TYPE_SIGNAL */
-# define signal_t RETSIGTYPE
-#else /* Imake */
-#ifdef SIGNALRETURNSINT
-#define signal_t int
-#else
-#define signal_t void
-#endif
-#endif /* RETSIGTYPE */
-static signal_t nameserver_lost(int sig);
+static void nameserver_lost(int sig);
 
 #define NAMESERVER_TIMEOUT 5	/* time to wait for nameserver */
 
@@ -149,7 +134,7 @@ static char *ProgramName;
 static int 
 XFamily(int af)
 {
-    int i;
+    unsigned int i;
     static struct _familyMap {
 	int af, xf;
     } familyMap[] = {
@@ -175,26 +160,24 @@ XFamily(int af)
 }
 #endif /* NEEDSOCKETS */
 
-static Display *dpy;
-
 int
 main(int argc, char *argv[])
 {
     register char *arg;
     int i, nhosts = 0;
-    char *hostname;
+    const char *hostname;
     int nfailed = 0;
     XHostAddress *list;
     Bool enabled = False;
-#ifdef DNETCONN
-    char *dnet_htoa();
-    struct nodeent *np;
-    struct dn_naddr *nlist, dnaddr, *dnaddrp, *dnet_addr();
-    char *cp;
-#endif
+    Display *dpy;
  
     ProgramName = argv[0];
 
+    if (argc == 2 && !strcmp(argv[1], "-help")) {
+	fprintf(stderr, "usage: %s [[+-]hostname ...]\n", argv[0]);
+	exit(1);
+    }
+
     if ((dpy = XOpenDisplay(NULL)) == NULL) {
 	fprintf(stderr, "%s:  unable to open display \"%s\"\n",
 		ProgramName, XDisplayName (NULL));
@@ -205,9 +188,6 @@ main(int argc, char *argv[])
  
  
     if (argc == 1) {
-#ifdef DNETCONN
-	setnodeent(1);		/* keep the database accessed */
-#endif
 	sethostent(1);		/* don't close the data base each time */
 	list = XListHosts(dpy, &nhosts, &enabled);
 	if (enabled)
@@ -263,11 +243,6 @@ main(int argc, char *argv[])
 	exit(0);
     }
  
-    if (argc == 2 && !strcmp(argv[1], "-help")) {
-	fprintf(stderr, "usage: %s [[+-]hostname ...]\n", argv[0]);
-	exit(1);
-    }
-
     for (i = 1; i < argc; i++) {
 	arg = argv[i];
 	if (*arg == '-') {
@@ -318,7 +293,8 @@ change_host(Display *dpy, char *name, Bool add)
 {
     XHostAddress ha;
     char *lname;
-    int namelen, i, family = FamilyWild;
+    size_t namelen, i;
+    int family = FamilyWild;
 #ifdef K5AUTH
     krb5_principal princ;
     krb5_data kbuf;
@@ -332,13 +308,8 @@ change_host(Display *dpy, char *name, Bool add)
 #endif
 #endif
     char *cp;
-#ifdef DNETCONN
-    struct dn_naddr *dnaddrp;
-    struct nodeent *np;
-    static struct dn_naddr dnaddr;
-#endif				/* DNETCONN */
-    static char *add_msg = "being added to access control list";
-    static char *remove_msg = "being removed from access control list";
+    static const char *add_msg = "being added to access control list";
+    static const char *remove_msg = "being removed from access control list";
 
     namelen = strlen(name);
     if ((lname = (char *)malloc(namelen+1)) == NULL) {
@@ -350,7 +321,7 @@ change_host(Display *dpy, char *name, Bool add)
     }
     lname[namelen] = '\0';
     if (!strncmp("inet:", lname, 5)) {
-#if defined(TCPCONN) || defined(STREAMSCONN)
+#ifdef TCPCONN
 	family = FamilyInternet;
 	name += 5;
 #else
@@ -360,8 +331,7 @@ change_host(Display *dpy, char *name, Bool add)
 #endif
     }
     else if (!strncmp("inet6:", lname, 6)) {
-#if (defined(TCPCONN) || defined(STREAMSCONN)) && \
-    defined(IPv6) && defined(AF_INET6)
+#if defined(TCPCONN) && defined(IPv6) && defined(AF_INET6)
 	family = FamilyInternet6;
 	name += 6;
 #else
@@ -373,8 +343,7 @@ change_host(Display *dpy, char *name, Bool add)
 #ifdef ACCEPT_INETV6 /* Allow inetv6 as an alias for inet6 for compatibility
 			with original X11 over IPv6 draft. */
     else if (!strncmp("inetv6:", lname, 7)) {
-#if (defined(TCPCONN) || defined(STREAMSCONN)) && \
-    defined(IPv6) && defined(AF_INET6)
+#if defined(TCPCONN) && defined(IPv6) && defined(AF_INET6)
 	family = FamilyInternet6;
 	name += 7;
 #else
@@ -385,14 +354,9 @@ change_host(Display *dpy, char *name, Bool add)
     }
 #endif /* ACCEPT_INETV6 */
     else if (!strncmp("dnet:", lname, 5)) {
-#ifdef DNETCONN
-	family = FamilyDECnet;
-	name += 5;
-#else
 	fprintf (stderr, "%s: not compiled for DECnet\n", ProgramName);
 	free(lname);
 	return 0;
-#endif
     }
     else if (!strncmp("nis:", lname, 4)) {
 #ifdef SECURE_RPC
@@ -432,7 +396,7 @@ change_host(Display *dpy, char *name, Bool add)
 
     if (family == FamilyServerInterpreted) {
 	XServerInterpretedAddress siaddr;
-	int namelen;
+	int rc;
 
 	cp = strchr(name, ':');
 	if (cp == NULL || cp == name) {
@@ -441,55 +405,23 @@ change_host(Display *dpy, char *name, Bool add)
 	      ProgramName, name);
 	    return 0;
 	}
+	siaddr.type = name;
+	siaddr.typelength = cp - name;
+	siaddr.value = ++cp;
+	siaddr.valuelength = strlen(cp);
 	ha.family = FamilyServerInterpreted;
 	ha.address = (char *) &siaddr;
-	namelen = strlen(name);
-	siaddr.type = malloc(namelen);
-	if (siaddr.type == NULL) {
-	    return 0;
-	}
-	memcpy(siaddr.type, name, namelen);
-	siaddr.typelength = cp - name;
-	siaddr.type[siaddr.typelength] = '\0';
-	siaddr.value = siaddr.type + siaddr.typelength + 1;
-	siaddr.valuelength = namelen - (siaddr.typelength + 1);
 	if (add)
-	    XAddHost(dpy, &ha);
+	    rc = XAddHost(dpy, &ha);
 	else
-	    XRemoveHost(dpy, &ha);
-	free(siaddr.type);
-	printf( "%s %s\n", name, add ? add_msg : remove_msg);
+	    rc = XRemoveHost(dpy, &ha);
+	printf( "%s %s%s\n", name, rc == 1 ? "" : "failed when ",
+		add ? add_msg : remove_msg);
+	if (rc != 1)
+	    return 0;
 	return 1;
     }
 
-#ifdef DNETCONN
-    if (family == FamilyDECnet || ((family == FamilyWild) &&
-	(cp = strchr(name, ':')) && (*(cp + 1) == ':') &&
-	!(*cp = '\0'))) {
-	ha.family = FamilyDECnet;
-	if (dnaddrp = dnet_addr(name)) {
-	    dnaddr = *dnaddrp;
-	} else {
-	    if ((np = getnodebyname (name)) == NULL) {
-		fprintf (stderr, "%s:  unable to get node name for \"%s::\"\n",
-			 ProgramName, name);
-		return 0;
-	    }
-	    dnaddr.a_len = np->n_length;
-	    memmove( dnaddr.a_addr, np->n_addr, np->n_length);
-	}
-	ha.length = sizeof(struct dn_naddr);
-	ha.address = (char *)&dnaddr;
-	if (add) {
-	    XAddHost (dpy, &ha);
-	    printf ("%s:: %s\n", name, add_msg);
-	} else {
-	    XRemoveHost (dpy, &ha);
-	    printf ("%s:: %s\n", name, remove_msg);
-	}
-	return 1;
-    }
-#endif				/* DNETCONN */
 #ifdef K5AUTH
     if (family == FamilyKrb5Principal) {
 	krb5_error_code retval;
@@ -516,8 +448,9 @@ change_host(Display *dpy, char *name, Bool add)
     }
 #endif
     if (family == FamilyLocalHost) {
+	char empty[] = "";
 	ha.length = 0;
-	ha.address = "";
+	ha.address = empty;
 	ha.family = family;
 	if (add)
 	    XAddHost(dpy, &ha);
@@ -708,37 +641,29 @@ change_host(Display *dpy, char *name, Bool add)
  * be found.
  */
 
-#ifdef X_NOT_POSIX
-jmp_buf env;
-#endif
 
-static char *
+static const char *
 get_hostname(XHostAddress *ha)
 {
-#if (defined(TCPCONN) || defined(STREAMSCONN)) &&	\
-     (!defined(IPv6) || !defined(AF_INET6))
+#if defined(TCPCONN) && (!defined(IPv6) || !defined(AF_INET6))
     static struct hostent *hp = NULL;
 #endif
-#ifdef DNETCONN
-    struct nodeent *np;
-    static char nodeaddr[5 + 2 * DN_MAXADDL];
-#endif				/* DNETCONN */
 #ifdef K5AUTH
     krb5_principal princ;
     krb5_data kbuf;
     char *kname;
     static char kname_out[255];
 #endif
-#ifndef X_NOT_POSIX
+#ifdef SIGALRM
     struct sigaction sa;
 #endif
 
-#if defined(TCPCONN) || defined(STREAMSCONN)
+#ifdef TCPCONN
 #if defined(IPv6) && defined(AF_INET6)
     if ((ha->family == FamilyInternet) || (ha->family == FamilyInternet6)) {
 	struct sockaddr_storage saddr;
 	static char inetname[NI_MAXHOST];
-	int saddrlen;
+	unsigned int saddrlen;
 
 	inetname[0] = '\0';
 	memset(&saddr, 0, sizeof saddr);
@@ -768,23 +693,18 @@ get_hostname(XHostAddress *ha)
 	   gethostbyaddr will continue after a signal, so we have to
 	   jump out of it. 
 	   */
-#ifndef X_NOT_POSIX
+#ifdef SIGALRM
 	memset(&sa, 0, sizeof sa);
 	sa.sa_handler = nameserver_lost;
 	sa.sa_flags = 0;	/* don't restart syscalls */
 	sigaction(SIGALRM, &sa, NULL);
-#else
-	signal(SIGALRM, nameserver_lost);
-#endif
 	alarm(NAMESERVER_TIMEOUT);
-#ifdef X_NOT_POSIX
-	if (setjmp(env) == 0) 
 #endif
-	{ 
-	    getnameinfo((struct sockaddr *) &saddr, saddrlen, inetname,
-	      sizeof(inetname), NULL, 0, 0);
-	}
+	getnameinfo((struct sockaddr *) &saddr, saddrlen, inetname,
+		    sizeof(inetname), NULL, 0, 0);
+#ifdef SIGALRM
 	alarm(0);
+#endif
 	if (nameserver_timedout || inetname[0] == '\0')
 	    inet_ntop(((struct sockaddr *)&saddr)->sa_family, ha->address,
 		inetname, sizeof(inetname));
@@ -792,35 +712,23 @@ get_hostname(XHostAddress *ha)
     }
 #else
     if (ha->family == FamilyInternet) {
-#ifdef CRAY
-	struct in_addr t_addr;
-	bzero((char *)&t_addr, sizeof(t_addr));
-	bcopy(ha->address, (char *)&t_addr, 4);
-	ha->address = (char *)&t_addr;
-#endif
 	/* gethostbyaddr can take a LONG time if the host does not exist.
 	   Assume that if it does not respond in NAMESERVER_TIMEOUT seconds
 	   that something is wrong and do not make the user wait.
 	   gethostbyaddr will continue after a signal, so we have to
 	   jump out of it. 
 	   */
-#ifndef X_NOT_POSIX
+#ifdef SIGALRM
 	memset(&sa, 0, sizeof sa);
 	sa.sa_handler = nameserver_lost;
 	sa.sa_flags = 0;	/* don't restart syscalls */
 	sigaction(SIGALRM, &sa, NULL);
-#else
-	signal(SIGALRM, nameserver_lost);
-#endif
 	alarm(4);
-#ifdef X_NOT_POSIX
-	if (setjmp(env) == 0) {
-#endif
-	    hp = gethostbyaddr (ha->address, ha->length, AF_INET);
-#ifdef X_NOT_POSIX
-	}
 #endif
+	hp = gethostbyaddr (ha->address, ha->length, AF_INET);
+#ifdef SIGALRM
 	alarm(0);
+#endif
 	if (hp)
 	    return (hp->h_name);
 	else return (inet_ntoa(*((struct in_addr *)(ha->address))));
@@ -848,24 +756,12 @@ get_hostname(XHostAddress *ha)
 
 	    pwd = getpwuid(uid);
 	    if (pwd)
-		sprintf(netname, "%s@ (%*.*s)", pwd->pw_name,
-			ha->length, ha->length, ha->address);
+		snprintf(netname, sizeof(netname), "%s@ (%*.*s)",
+			 pwd->pw_name, ha->length, ha->length, ha->address);
 	}
 #endif
 	return (netname);
     }
-#ifdef DNETCONN
-    if (ha->family == FamilyDECnet) {
-	struct dn_naddr *addr_ptr = (struct dn_naddr *) ha->address;
-
-	if (np = getnodebyaddr(addr_ptr->a_addr, addr_ptr->a_len, AF_DECnet)) {
-	    sprintf(nodeaddr, "%s", np->n_name);
-	} else {
-	    sprintf(nodeaddr, "%s", dnet_htoa(ha->address));
-	}
-	return(nodeaddr);
-    }
-#endif
 #ifdef K5AUTH
     if (ha->family == FamilyKrb5Principal) {
 	kbuf.data = ha->address;
@@ -913,15 +809,10 @@ get_hostname(XHostAddress *ha)
 }
 
 /*ARGUSED*/
-static signal_t 
-nameserver_lost(int sig)
+static void
+nameserver_lost(_X_UNUSED int sig)
 {
     nameserver_timedout = 1;
-#ifdef X_NOT_POSIX
-    /* not needed with POSIX signals - stuck syscalls will not 
-       be restarted after signal delivery */
-    longjmp(env, -1);
-#endif
 }
 
 /*
@@ -952,7 +843,7 @@ local_xerror(Display *dpy, XErrorEvent *rep)
     return 0;
 }
 
-#ifdef __CYGWIN__
+#if defined(__CYGWIN__) || defined(WIN32)
 void sethostent(int x)
 {}
 
